@@ -1,4 +1,4 @@
-// Copyright 2018-2024 the Deno authors. MIT license.
+// Copyright 2018-2025 the Deno authors. MIT license.
 
 #![deny(clippy::print_stderr)]
 #![deny(clippy::print_stdout)]
@@ -6,6 +6,7 @@
 #![deny(clippy::unnecessary_wraps)]
 
 use deno_error::JsError;
+use std::borrow::Cow;
 use std::path::Component;
 use std::path::Path;
 use std::path::PathBuf;
@@ -140,10 +141,79 @@ fn url_to_file_path_wasm(url: &Url) -> Result<PathBuf, ()> {
 /// Normalize all intermediate components of the path (ie. remove "./" and "../" components).
 /// Similar to `fs::canonicalize()` but doesn't resolve symlinks.
 ///
-/// Taken from Cargo
+/// Adapted from Cargo
 /// <https://github.com/rust-lang/cargo/blob/af307a38c20a753ec60f0ad18be5abed3db3c9ac/src/cargo/util/paths.rs#L60-L85>
 #[inline]
-pub fn normalize_path<P: AsRef<Path>>(path: P) -> PathBuf {
+pub fn normalize_path(path: Cow<Path>) -> Cow<Path> {
+  fn should_normalize(path: &Path) -> bool {
+    if path_has_trailing_separator(path) {
+      return true;
+    }
+
+    for component in path.components() {
+      match component {
+        Component::CurDir | Component::ParentDir => {
+          return true;
+        }
+        Component::Prefix(..) | Component::RootDir | Component::Normal(_) => {
+          // ok
+        }
+      }
+    }
+
+    path_has_cur_dir_separator(path)
+  }
+
+  fn path_has_trailing_separator(path: &Path) -> bool {
+    #[cfg(unix)]
+    let raw = std::os::unix::ffi::OsStrExt::as_bytes(path.as_os_str());
+    #[cfg(windows)]
+    let raw = path.as_os_str().as_encoded_bytes();
+    #[cfg(target_arch = "wasm32")]
+    let raw = path.to_string_lossy();
+    #[cfg(target_arch = "wasm32")]
+    let raw = raw.as_bytes();
+
+    if sys_traits::impls::is_windows() {
+      raw.contains(&b'/') || raw.ends_with(b"\\")
+    } else {
+      raw.ends_with(b"/")
+    }
+  }
+
+  // Rust normalizes away `Component::CurDir` most of the time
+  // so we need to explicitly check for it in the bytes
+  fn path_has_cur_dir_separator(path: &Path) -> bool {
+    #[cfg(unix)]
+    let raw = std::os::unix::ffi::OsStrExt::as_bytes(path.as_os_str());
+    #[cfg(windows)]
+    let raw = path.as_os_str().as_encoded_bytes();
+    #[cfg(target_arch = "wasm32")]
+    let raw = path.to_string_lossy();
+    #[cfg(target_arch = "wasm32")]
+    let raw = raw.as_bytes();
+
+    if raw.ends_with(b"\\.") || raw.ends_with(b"/.") {
+      return true;
+    }
+
+    if sys_traits::impls::is_windows() {
+      for window in raw.windows(3) {
+        if matches!(window, [b'\\', b'.', b'\\']) {
+          return true;
+        }
+      }
+    } else {
+      for window in raw.windows(3) {
+        if matches!(window, [b'/', b'.', b'/']) {
+          return true;
+        }
+      }
+    }
+
+    false
+  }
+
   fn inner(path: &Path) -> PathBuf {
     let mut components = path.components().peekable();
     let mut ret =
@@ -172,7 +242,11 @@ pub fn normalize_path<P: AsRef<Path>>(path: P) -> PathBuf {
     ret
   }
 
-  inner(path.as_ref())
+  if should_normalize(&path) {
+    Cow::Owned(inner(&path))
+  } else {
+    path
+  }
 }
 
 #[derive(Debug, Clone, Error, deno_error::JsError, PartialEq, Eq)]
@@ -182,20 +256,23 @@ pub struct PathToUrlError(pub PathBuf);
 
 #[allow(clippy::result_unit_err)]
 pub fn url_from_file_path(path: &Path) -> Result<Url, PathToUrlError> {
+  let path = normalize_path(Cow::Borrowed(path));
   #[cfg(any(unix, windows, target_os = "redox", target_os = "wasi"))]
-  return Url::from_file_path(path)
+  return Url::from_file_path(&path)
     .map_err(|()| PathToUrlError(path.to_path_buf()));
   #[cfg(not(any(unix, windows, target_os = "redox", target_os = "wasi")))]
-  url_from_file_path_wasm(path).map_err(|()| PathToUrlError(path.to_path_buf()))
+  url_from_file_path_wasm(&path)
+    .map_err(|()| PathToUrlError(path.to_path_buf()))
 }
 
 #[allow(clippy::result_unit_err)]
 pub fn url_from_directory_path(path: &Path) -> Result<Url, PathToUrlError> {
+  let path = normalize_path(Cow::Borrowed(path));
   #[cfg(any(unix, windows, target_os = "redox", target_os = "wasi"))]
-  return Url::from_directory_path(path)
+  return Url::from_directory_path(&path)
     .map_err(|()| PathToUrlError(path.to_path_buf()));
   #[cfg(not(any(unix, windows, target_os = "redox", target_os = "wasi")))]
-  url_from_directory_path_wasm(path)
+  url_from_directory_path_wasm(&path)
     .map_err(|()| PathToUrlError(path.to_path_buf()))
 }
 
@@ -378,7 +455,6 @@ pub fn resolve_path(
   current_dir: &Path,
 ) -> Result<Url, PathToUrlError> {
   let path = current_dir.join(path_str);
-  let path = normalize_path(path);
   url_from_file_path(&path)
 }
 
@@ -440,6 +516,7 @@ mod tests {
     run_success_test("file:///", "/");
     run_success_test("file:///test", "/test");
     run_success_test("file:///dir/test/test.txt", "/dir/test/test.txt");
+    #[cfg(not(debug_assertions))]
     run_success_test(
       "file:///dir/test%20test/test.txt",
       "/dir/test test/test.txt",
@@ -447,14 +524,17 @@ mod tests {
 
     assert_no_panic_url_to_file_path("file:/");
     assert_no_panic_url_to_file_path("file://");
+    #[cfg(not(debug_assertions))]
     assert_no_panic_url_to_file_path("file://asdf/");
     assert_no_panic_url_to_file_path("file://asdf/66666/a.ts");
 
+    #[track_caller]
     fn run_success_test(url: &str, expected_path: &str) {
       let result = url_to_file_path(&Url::parse(url).unwrap()).unwrap();
       assert_eq!(result, PathBuf::from(expected_path));
     }
 
+    #[track_caller]
     fn assert_no_panic_url_to_file_path(url: &str) {
       let _result = url_to_file_path(&Url::parse(url).unwrap());
     }
@@ -578,21 +658,40 @@ mod tests {
     }
   }
 
+  #[test]
+  fn test_normalize_path_basic() {
+    let run_test = run_normalize_path_test;
+    run_test("a/../b", "b");
+    run_test("a/./b/", &PathBuf::from("a").join("b").to_string_lossy());
+    run_test(
+      "a/./b/../c",
+      &PathBuf::from("a").join("c").to_string_lossy(),
+    );
+  }
+
   #[cfg(windows)]
   #[test]
-  fn test_normalize_path() {
-    use super::*;
+  fn test_normalize_path_win() {
+    let run_test = run_normalize_path_test;
 
+    run_test("C:\\test\\file.txt", "C:\\test\\file.txt");
     run_test("C:\\test\\./file.txt", "C:\\test\\file.txt");
     run_test("C:\\test\\../other/file.txt", "C:\\other\\file.txt");
     run_test("C:\\test\\../other\\file.txt", "C:\\other\\file.txt");
+    run_test(
+      "C:\\test\\removes_trailing_slash\\",
+      "C:\\test\\removes_trailing_slash",
+    );
+    run_test("C:\\a\\.\\b\\..\\c", "C:\\a\\c");
+    run_test("C:\\test\\.", "C:\\test");
+  }
 
-    fn run_test(input: &str, expected: &str) {
-      assert_eq!(
-        normalize_path(PathBuf::from(input)),
-        PathBuf::from(expected)
-      );
-    }
+  #[track_caller]
+  fn run_normalize_path_test(input: &str, expected: &str) {
+    assert_eq!(
+      normalize_path(Cow::Owned(PathBuf::from(input))).to_string_lossy(),
+      expected
+    );
   }
 
   #[test]
