@@ -1,10 +1,9 @@
 use crate::faker::automotive::raw::*;
-use crate::locales::FR_FR;
+use crate::locales::{FR_FR, IT_IT};
 use crate::{Dummy, Fake};
-use rand::prelude::SliceRandom;
+use rand::seq::IndexedRandom;
 use rand::Rng;
 use std::char;
-
 /* ABC without I, O and U
 As with the SIV system, The letters I and O were never used because they could be confused with other characters, like 1 and 0.
 ref https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_France
@@ -18,7 +17,7 @@ const LICENSE_CHARS: [char; 23] = [
 ];
 
 #[inline]
-fn numerify_licence_plate<R: Rng + ?Sized>(string: &str, rng: &mut R) -> String {
+pub(crate) fn numerify_licence_plate<R: Rng + ?Sized>(string: &str, rng: &mut R) -> String {
     string
         .chars()
         .map(|x| match x {
@@ -37,3 +36,10 @@ impl Dummy<LicencePlate<FR_FR>> for String {
         numerify_licence_plate(fmt, rng)
     }
 }
+
+impl Dummy<LicencePlate<IT_IT>> for String {
+    fn dummy_with_rng<R: rand::Rng + ?Sized>(_: &LicencePlate<IT_IT>, rng: &mut R) -> Self {
+        let fmt = LICENSE_PLATE.choose(rng).unwrap();
+        crate::faker::impls::automotive::numerify_licence_plate(fmt, rng)
+    }
+}
