@@ -510,7 +510,7 @@ impl<'de> serde::de::Deserialize<'de> for Duration {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use serde_json::json;
     use test_case::test_case;
@@ -694,7 +694,7 @@ mod test {
         let got = serde_json::from_value::<Duration>(serde_json::json!({}));
         assert!(got.is_err());
         let msg = format!("{got:?}");
-        assert!(msg.contains("duration in Google format"), "message={}", msg);
+        assert!(msg.contains("duration in Google format"), "message={msg}");
         Ok(())
     }
 
