@@ -14,7 +14,7 @@ it:
 use jiff::{Timestamp, ToSpan};
 
 let time: Timestamp = "2024-07-11T01:14:00Z".parse()?;
-let zoned = time.intz("America/New_York")?.checked_add(1.month().hours(2))?;
+let zoned = time.in_tz("America/New_York")?.checked_add(1.month().hours(2))?;
 assert_eq!(zoned.to_string(), "2024-08-10T23:14:00-04:00[America/New_York]");
 // Or, if you want an RFC3339 formatted string:
 assert_eq!(zoned.timestamp().to_string(), "2024-08-11T03:14:00Z");
@@ -44,9 +44,10 @@ All three of these components are used to provide convenient high level
 operations on `Zoned` such as computing durations, adding durations and
 rounding.
 
-This crate defines a single duration type, [`Span`], which mixes calendar
-and clock units into a single type. All datetime types in Jiff use spans for
-operating on durations.
+A [`Span`] is this crate's primary duration type. It mixes calendar and clock
+units into a single type. Jiff also provides [`SignedDuration`], which is like
+[`std::time::Duration`], but signed. Users should default to a `Span` for
+representing durations when using Jiff.
 
 [Unix epoch]: https://en.wikipedia.org/wiki/Unix_time
 [UTC]: https://en.wikipedia.org/wiki/Coordinated_Universal_Time
@@ -117,8 +118,8 @@ maximum datetime values.][cppchrono]
 * [Jiff aims to have reasonable performance and may not be capable of doing the
 fastest possible thing.][perf]
 
-At present, it is recommended to use the [`icu`] crate for localization and
-non-Gregorian use cases.
+At present, it is recommended to use the [`icu`] crate via [`jiff-icu`] for
+localization and non-Gregorian use cases.
 
 [Leap seconds]: https://github.com/BurntSushi/jiff/issues/7
 [Calendars other than Gregorian]: https://github.com/BurntSushi/jiff/issues/6
@@ -126,6 +127,7 @@ non-Gregorian use cases.
 [cppchrono]: https://github.com/BurntSushi/jiff/issues/3
 [perf]: https://github.com/BurntSushi/jiff/issues/17
 [`icu`]: https://docs.rs/icu
+[`jiff-icu`]: https://docs.rs/jiff-icu
 
 Please file an issue if you can think of more (substantial) things to add to
 the above list.
@@ -174,8 +176,8 @@ Fourth, run it with `cargo run`:
 
 ```text
 $ cargo run
-   Compiling jiff v0.1.0 (/home/andrew/rust/jiff)
-   Compiling jiff-play v0.1.0 (/home/andrew/tmp/scratch/rust/jiff-play)
+   Compiling jiff v0.2.0 (/home/andrew/rust/jiff)
+   Compiling jiff-play v0.2.0 (/home/andrew/tmp/scratch/rust/jiff-play)
     Finished `dev` profile [unoptimized + debuginfo] target(s) in 1.37s
      Running `target/debug/jiff-play`
 2024-07-10T19:54:20-04:00[America/New_York]
@@ -323,13 +325,13 @@ assert_eq!(ts.to_string(), "2024-07-10T21:19:25.567Z");
 This example demonstrates the convenience constructor, [`civil::date`],
 for a [`civil::Date`]. And use the [`civil::Date::at`] method to create
 a [`civil::DateTime`]. Once we have a civil datetime, we can use
-[`civil::DateTime::intz`] to do a time zone lookup and convert it to a precise
+[`civil::DateTime::in_tz`] to do a time zone lookup and convert it to a precise
 instant in time:
 
 ```
 use jiff::civil::date;
 
-let zdt = date(2023, 12, 31).at(18, 30, 0, 0).intz("America/New_York")?;
+let zdt = date(2023, 12, 31).at(18, 30, 0, 0).in_tz("America/New_York")?;
 assert_eq!(zdt.to_string(), "2023-12-31T18:30:00-05:00[America/New_York]");
 
 # Ok::<(), Box<dyn std::error::Error>>(())
@@ -346,8 +348,8 @@ This shows how to find the civil time, in New York, when World War 1 ended:
 ```
 use jiff::civil::date;
 
-let zdt1 = date(1918, 11, 11).at(11, 0, 0, 0).intz("Europe/Paris")?;
-let zdt2 = zdt1.intz("America/New_York")?;
+let zdt1 = date(1918, 11, 11).at(11, 0, 0, 0).in_tz("Europe/Paris")?;
+let zdt2 = zdt1.in_tz("America/New_York")?;
 assert_eq!(
     zdt2.to_string(),
     "1918-11-11T06:00:00-05:00[America/New_York]",
@@ -365,8 +367,8 @@ datetimes via the `-` operator:
 ```
 use jiff::civil::date;
 
-let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).intz("America/New_York")?;
-let zdt2 = date(2023, 12, 31).at(18, 30, 0, 0).intz("America/New_York")?;
+let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")?;
+let zdt2 = date(2023, 12, 31).at(18, 30, 0, 0).in_tz("America/New_York")?;
 let span = &zdt2 - &zdt1;
 assert_eq!(format!("{span:#}"), "29341h 3m");
 
@@ -381,8 +383,8 @@ via [`Zoned::until`] to make the span more comprehensible:
 ```
 use jiff::{civil::date, Unit};
 
-let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).intz("America/New_York")?;
-let zdt2 = date(2023, 12, 31).at(18, 30, 0, 0).intz("America/New_York")?;
+let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")?;
+let zdt2 = date(2023, 12, 31).at(18, 30, 0, 0).in_tz("America/New_York")?;
 let span = zdt1.until((Unit::Year, &zdt2))?;
 assert_eq!(format!("{span:#}"), "3y 4mo 5d 12h 3m");
 
@@ -398,7 +400,7 @@ trait for convenience construction of `Span` values.
 ```
 use jiff::{civil::date, ToSpan};
 
-let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).intz("America/New_York")?;
+let zdt1 = date(2020, 8, 26).at(6, 27, 0, 0).in_tz("America/New_York")?;
 let span = 3.years().months(4).days(5).hours(12).minutes(3);
 let zdt2 = zdt1.checked_add(span)?;
 assert_eq!(zdt2.to_string(), "2023-12-31T18:30:00-05:00[America/New_York]");
@@ -422,12 +424,12 @@ use jiff::civil::date;
 
 // 2:30 on 2024-03-10 in New York didn't exist. It's a "gap."
 // The compatible strategy selects the datetime after the gap.
-let zdt = date(2024, 3, 10).at(2, 30, 0, 0).intz("America/New_York")?;
+let zdt = date(2024, 3, 10).at(2, 30, 0, 0).in_tz("America/New_York")?;
 assert_eq!(zdt.to_string(), "2024-03-10T03:30:00-04:00[America/New_York]");
 
 // 1:30 on 2024-11-03 in New York appeared twice. It's a "fold."
 // The compatible strategy selects the datetime before the fold.
-let zdt = date(2024, 11, 3).at(1, 30, 0, 0).intz("America/New_York")?;
+let zdt = date(2024, 11, 3).at(1, 30, 0, 0).in_tz("America/New_York")?;
 assert_eq!(zdt.to_string(), "2024-11-03T01:30:00-04:00[America/New_York]");
 
 # Ok::<(), Box<dyn std::error::Error>>(())
@@ -447,7 +449,7 @@ use jiff::Span;
 
 let span: Span = "P5y1w10dT5h59m".parse()?;
 let expected = Span::new().years(5).weeks(1).days(10).hours(5).minutes(59);
-assert_eq!(span, expected);
+assert_eq!(span, expected.fieldwise());
 
 # Ok::<(), Box<dyn std::error::Error>>(())
 ```
@@ -463,11 +465,11 @@ use jiff::Span;
 
 let expected = Span::new().years(5).weeks(1).days(10).hours(5).minutes(59);
 let span: Span = "5 years, 1 week, 10 days, 5 hours, 59 minutes".parse()?;
-assert_eq!(span, expected);
+assert_eq!(span, expected.fieldwise());
 let span: Span = "5yrs 1wk 10d 5hrs 59mins".parse()?;
-assert_eq!(span, expected);
+assert_eq!(span, expected.fieldwise());
 let span: Span = "5y 1w 10d 5h 59m".parse()?;
-assert_eq!(span, expected);
+assert_eq!(span, expected.fieldwise());
 
 # Ok::<(), Box<dyn std::error::Error>>(())
 ```
@@ -483,9 +485,9 @@ or email). Parsing and printing of RFC 2822 datetimes is done via the
 use jiff::fmt::rfc2822;
 
 let zdt1 = rfc2822::parse("Thu, 29 Feb 2024 05:34 -0500")?;
-let zdt2 = zdt1.intz("Australia/Tasmania")?;
+let zdt2 = zdt1.in_tz("Australia/Tasmania")?;
 assert_eq!(rfc2822::to_string(&zdt2)?, "Thu, 29 Feb 2024 21:34:00 +1100");
-let zdt3 = zdt1.intz("Asia/Kolkata")?;
+let zdt3 = zdt1.in_tz("Asia/Kolkata")?;
 assert_eq!(rfc2822::to_string(&zdt3)?, "Thu, 29 Feb 2024 16:04:00 +0530");
 
 # Ok::<(), Box<dyn std::error::Error>>(())
@@ -505,7 +507,7 @@ a "odd" custom format into a zoned datetime:
 use jiff::Zoned;
 
 let zdt = Zoned::strptime(
-    "%A, %B %d, %Y at %I:%M%p %V",
+    "%A, %B %d, %Y at %I:%M%p %Q",
     "Monday, July 15, 2024 at 5:30pm US/Eastern",
 )?;
 assert_eq!(zdt.to_string(), "2024-07-15T17:30:00-04:00[US/Eastern]");
@@ -520,7 +522,7 @@ available) instead of an offset (`%Z` can't be used for parsing):
 ```
 use jiff::civil::date;
 
-let zdt = date(2024, 7, 15).at(17, 30, 59, 0).intz("Australia/Tasmania")?;
+let zdt = date(2024, 7, 15).at(17, 30, 59, 0).in_tz("Australia/Tasmania")?;
 // %-I instead of %I means no padding.
 let string = zdt.strftime("%A, %B %d, %Y at %-I:%M%P %Z").to_string();
 assert_eq!(string, "Monday, July 15, 2024 at 5:30pm AEST");
@@ -530,15 +532,15 @@ assert_eq!(string, "Monday, July 15, 2024 at 5:30pm AEST");
 
 However, time zone abbreviations aren't parsable because they are ambiguous.
 For example, `CST` can stand for `Central Standard Time`, `Cuba Standard Time`
-or `China Standard Time`. Instead, it is recommended to use `%V` to format an
+or `China Standard Time`. Instead, it is recommended to use `%Q` to format an
 IANA time zone identifier (which can be parsed, as shown above):
 
 ```
 use jiff::civil::date;
 
-let zdt = date(2024, 7, 15).at(17, 30, 59, 0).intz("Australia/Tasmania")?;
+let zdt = date(2024, 7, 15).at(17, 30, 59, 0).in_tz("Australia/Tasmania")?;
 // %-I instead of %I means no padding.
-let string = zdt.strftime("%A, %B %d, %Y at %-I:%M%P %V").to_string();
+let string = zdt.strftime("%A, %B %d, %Y at %-I:%M%P %Q").to_string();
 assert_eq!(string, "Monday, July 15, 2024 at 5:30pm Australia/Tasmania");
 
 # Ok::<(), Box<dyn std::error::Error>>(())
@@ -595,7 +597,7 @@ For more, see the [`fmt::serde`] sub-module. (This requires enabling Jiff's
   bundle the Time Zone Database), then Jiff has nearly full functionality
   without `std` enabled, excepting things like `std::error::Error` trait
   implementations and a global time zone database (which is required for
-  things like [`Timestamp::intz`] to work).
+  things like [`Timestamp::in_tz`] to work).
 * **alloc** (enabled by default) -
   When enabled, Jiff will depend on the `alloc` crate. In particular, this
   enables functionality that requires or greatly benefits from dynamic memory
@@ -636,6 +638,13 @@ For more, see the [`fmt::serde`] sub-module. (This requires enabling Jiff's
   looking at the symlink information on `/etc/localtime`. But in general, it's
   very platform specific and heuristic oriented. On some platforms, this may
   require extra dependencies. (For example, `windows-sys` on Windows.)
+* **tz-fat** (enabled by default) -
+  When enabled, Jiff will "fatten" time zone data with extra transitions to
+  make time zone lookups faster. This may result in increased heap memory
+  (when loading time zones from `/usr/share/zoneinfo`) or increased binary
+  size (when using the `jiff-static` proc macros). Note that this doesn't add
+  more transitions than are likely already in `/usr/share/zoneinfo`, depending
+  on how it was generated.
 * **tzdb-bundle-always** -
   When enabled, Jiff will forcefully depend on the `jiff-tzdb` crate, which
   embeds an entire copy of the Time Zone Database. You should avoid this unless
@@ -649,18 +658,68 @@ For more, see the [`fmt::serde`] sub-module. (This requires enabling Jiff's
 * **tzdb-zoneinfo** (enabled by default) -
   When enabled, Jiff will attempt to look for your system's copy of the Time
   Zone Database.
+* **tzdb-concatenated** (enabled by default) -
+  When enabled, Jiff will attempt to look for a system copy of the
+  [Concatenated Time Zone Database]. This is primarily meant for reading time
+  zone information on Android platforms. The `ANDROID_ROOT` and `ANDROID_DATA`
+  environment variables (with sensible default fallbacks) are used to construct
+  candidate paths to look for this database. For more on this, see the
+  [Android section of the platform support documentation](crate::_documentation::platform#android).
+* **static** -
+  When enabled, new procedural macros will be added to the `tz` sub-module for
+  creating static `TimeZone` values at compile-time. This adds a dependency on
+  [`jiff-static`] and [`jiff-tzdb`]. `jiff-static` defines the macros, and Jiff
+  re-exports them. This also enables `static-tz`.
+* **static-tz** -
+  When enabled, a `jiff::tz::include` procedural macro will become available.
+  This takes a TZif file path, like `/usr/share/zoneinfo/Israel`, as input and
+  returns a `TimeZone` value at compile time.
 
+### Performance features
+
+* **perf-inline** (enabled by default) -
+  When enabled, a number of `inline(always)` annotations are used inside of
+  Jiff to improve performance. This can especially impact formatting and
+  parsing of datetimes. If the extra performance isn't needed or if you want
+  to prioritize smaller binary sizes and shorter compilation times over
+  runtime performance, then it can be useful to disable this feature.
+
+[`jiff-static`]: https://docs.rs/jiff-static
+[`jiff-tzdb`]: https://docs.rs/jiff-tzdb
+[Concatenated Time Zone Database]: https://android.googlesource.com/platform/libcore/+/jb-mr2-release/luni/src/main/java/libcore/util/ZoneInfoDB.java
 */
 
 #![no_std]
 // Lots of rustdoc links break when disabling default features because docs
 // aren't written conditionally.
 #![cfg_attr(
-    all(feature = "std", feature = "serde", feature = "tzdb-zoneinfo"),
+    all(
+        feature = "std",
+        feature = "serde",
+        feature = "static",
+        feature = "tzdb-zoneinfo"
+    ),
     deny(rustdoc::broken_intra_doc_links)
 )]
+#![cfg_attr(
+    not(all(
+        feature = "std",
+        feature = "serde",
+        feature = "static",
+        feature = "tzdb-zoneinfo"
+    )),
+    allow(rustdoc::broken_intra_doc_links)
+)]
 // These are just too annoying to squash otherwise.
-#![cfg_attr(not(feature = "std"), allow(dead_code, unused_imports))]
+#![cfg_attr(
+    not(all(
+        feature = "std",
+        feature = "tzdb-zoneinfo",
+        feature = "tzdb-concatenated",
+        feature = "tz-system",
+    )),
+    allow(dead_code, unused_imports)
+)]
 // No clue why this thing is still unstable because it's pretty amazing. This
 // adds Cargo feature annotations to items in the rustdoc output. Which is
 // sadly hugely beneficial for this crate due to the number of features.
@@ -693,8 +752,8 @@ pub use crate::{
     error::Error,
     signed_duration::{SignedDuration, SignedDurationRound},
     span::{
-        Span, SpanArithmetic, SpanCompare, SpanRelativeTo, SpanRound,
-        SpanTotal, ToSpan, Unit,
+        Span, SpanArithmetic, SpanCompare, SpanFieldwise, SpanRelativeTo,
+        SpanRound, SpanTotal, ToSpan, Unit,
     },
     timestamp::{
         Timestamp, TimestampArithmetic, TimestampDifference,
@@ -713,6 +772,8 @@ mod error;
 pub mod fmt;
 #[cfg(feature = "std")]
 mod now;
+#[doc(hidden)]
+pub mod shared;
 mod signed_duration;
 mod span;
 mod timestamp;
@@ -747,7 +808,7 @@ mod tests {
 
     #[cfg(feature = "std")]
     #[test]
-    fn topscratch() {
+    fn ranges() {
         use crate::util::t;
 
         dbg!((t::SpanYears::MIN, t::SpanYears::MAX));
@@ -760,5 +821,42 @@ mod tests {
         dbg!((t::SpanMilliseconds::MIN, t::SpanMilliseconds::MAX));
         dbg!((t::SpanMicroseconds::MIN, t::SpanMicroseconds::MAX));
         dbg!((t::SpanNanoseconds::MIN, t::SpanNanoseconds::MAX));
+        dbg!((t::UnixSeconds::MIN, t::UnixSeconds::MAX));
+        dbg!((t::UnixEpochDay::MIN, t::UnixEpochDay::MAX));
+    }
+
+    #[cfg(feature = "std")]
+    #[test]
+    fn maximally_long_span() {
+        use crate::{fmt::friendly, util::t};
+
+        let span = Span::new()
+            .years(t::SpanYears::MAX_REPR)
+            .months(t::SpanMonths::MAX_REPR)
+            .weeks(t::SpanWeeks::MAX_REPR)
+            .days(t::SpanDays::MAX_REPR)
+            .hours(t::SpanHours::MAX_REPR)
+            .minutes(t::SpanMinutes::MAX_REPR)
+            .seconds(t::SpanSeconds::MAX_REPR)
+            .milliseconds(t::SpanMilliseconds::MAX_REPR)
+            .microseconds(t::SpanMicroseconds::MAX_REPR)
+            .nanoseconds(t::SpanNanoseconds::MAX_REPR)
+            .negate();
+        std::println!("{span}");
+        std::println!("{span:#}");
+
+        let long_printer = friendly::SpanPrinter::new()
+            .designator(friendly::Designator::Verbose)
+            .spacing(friendly::Spacing::BetweenUnitsAndDesignators)
+            .comma_after_designator(true);
+        std::println!("{}", long_printer.span_to_string(&span));
+    }
+
+    #[cfg(feature = "std")]
+    #[test]
+    fn maximally_long_duration() {
+        let sdur = SignedDuration::MIN;
+        std::println!("{sdur}");
+        std::println!("{sdur:#}");
     }
 }
