@@ -238,6 +238,13 @@ impl RecordFromPath for PackageRecord {
     }
 }
 
+impl PackageRecord {
+    /// Returns true if package `run_exports` is some.
+    pub fn has_run_exports(&self) -> bool {
+        self.run_exports.is_some()
+    }
+}
+
 impl RepoData {
     /// Parses [`RepoData`] from a file.
     pub fn from_path(path: impl AsRef<Path>) -> Result<Self, std::io::Error> {
@@ -409,6 +416,46 @@ impl PackageRecord {
     }
 }
 
+#[derive(Debug, Default, Deserialize, Serialize, Eq, PartialEq, Clone)]
+struct PackageRunExports {
+    run_exports: RunExportsJson,
+}
+
+/// Represents [`Channel`] global map from package file names to
+/// [`RunExportsJson`].
+///
+/// See [CEP 12](https://github.com/conda/ceps/blob/main/cep-0012.md) for more info.
+#[derive(Debug, Default, Deserialize, Serialize, Eq, PartialEq, Clone)]
+pub struct SubdirRunExportsJson {
+    info: Option<ChannelInfo>,
+
+    #[serde(default, serialize_with = "sort_map_alphabetically")]
+    packages: FxHashMap<String, PackageRunExports>,
+
+    #[serde(
+        default,
+        rename = "packages.conda",
+        serialize_with = "sort_map_alphabetically"
+    )]
+    conda_packages: FxHashMap<String, PackageRunExports>,
+}
+
+impl SubdirRunExportsJson {
+    /// Get package [`RunExportsJson`] based on the package file name.
+    pub fn get(&self, record: &RepoDataRecord) -> Option<&RunExportsJson> {
+        let file_name = &record.file_name;
+        self.packages
+            .get(file_name)
+            .or_else(|| self.conda_packages.get(file_name))
+            .map(|pre| &pre.run_exports)
+    }
+
+    /// Returns optional [`ChannelInfo`].
+    pub fn info(&self) -> Option<&ChannelInfo> {
+        self.info.as_ref()
+    }
+}
+
 /// An error when validating package records.
 #[derive(Debug, Error)]
 #[allow(clippy::large_enum_variant)]
@@ -591,6 +638,27 @@ mod test {
         insta::assert_yaml_snapshot!(repodata);
     }
 
+    #[test]
+    fn test_deserialize_no_noarch_empty_str() {
+        // This test covers the case where a repodata entry may contain a "noarch" key set to an empty string.
+        // Packages with such metadata have been observed on private conda channels.
+        // This likely was passed from older versions of conda-build that would pass this key
+        // from the recipe even if it was incorrect.
+        let repodata =
+            deserialize_json_from_test_data("channels/dummy-noarch-str/linux-64/repodata.json");
+        insta::assert_yaml_snapshot!(repodata);
+    }
+
+    #[test]
+    fn test_deserialize_no_noarch_not_empty_str_should_fail() {
+        let test_data_path =
+            std::path::Path::new(env!("CARGO_MANIFEST_DIR")).join("../../test-data");
+        let data_path =
+            test_data_path.join("channels/dummy-noarch-str-not-empty/linux-64/repodata.json");
+        let err = RepoData::from_path(data_path).unwrap_err();
+        insta::assert_snapshot!(err.to_string(), @r#"invalid value: string "notempty-this-should-fail", expected '' at line 26 column 43"#);
+    }
+
     #[test]
     fn test_base_url_packages() {
         // load test data
