@@ -7,6 +7,7 @@ use std::{
 
 use elsa::FrozenMap;
 
+use crate::solver::conditions::Disjunction;
 use crate::{
     Interner, NameId, Requirement,
     internal::{
@@ -14,8 +15,8 @@ use crate::{
         id::{ClauseId, LearntClauseId, StringId, VersionSetId},
     },
     solver::{
-        VariableId, decision_map::DecisionMap, decision_tracker::DecisionTracker,
-        variable_map::VariableMap,
+        VariableId, conditions::DisjunctionId, decision_map::DecisionMap,
+        decision_tracker::DecisionTracker, variable_map::VariableMap,
     },
 };
 
@@ -55,9 +56,13 @@ pub(crate) enum Clause {
     /// Makes the solvable require the candidates associated with the
     /// [`Requirement`].
     ///
-    /// In SAT terms: (¬A ∨ B1 ∨ B2 ∨ ... ∨ B99), where B1 to B99 represent the
-    /// possible candidates for the provided [`Requirement`].
-    Requires(VariableId, Requirement),
+    /// Optionally the requirement can be associated with a condition in the
+    /// form of a disjunction.
+    ///
+    /// In SAT terms: (¬A ∨ ¬D1 v ¬D2 .. v ¬D99 v B1 ∨ B2 ∨ ... ∨ B99), where D1
+    /// to D99 represent the candidates of the disjunction and B1 to B99
+    /// represent the possible candidates for the provided [`Requirement`].
+    Requires(VariableId, Option<DisjunctionId>, Requirement),
     /// Ensures only a single version of a package is installed
     ///
     /// Usage: generate one [`Clause::ForbidMultipleInstances`] clause for each
@@ -97,6 +102,10 @@ pub(crate) enum Clause {
     /// A clause that forbids a package from being installed for an external
     /// reason.
     Excluded(VariableId, StringId),
+
+    /// A clause that indicates that any version of a package C is selected.
+    /// In SAT terms: (C_selected v ¬Cj)
+    AnyOf(VariableId, VariableId),
 }
 
 impl Clause {
@@ -116,37 +125,47 @@ impl Clause {
         parent: VariableId,
         requirement: Requirement,
         candidates: impl IntoIterator<Item = VariableId>,
+        condition: Option<(DisjunctionId, &[Literal])>,
         decision_tracker: &DecisionTracker,
     ) -> (Self, Option<[Literal; 2]>, bool) {
         // It only makes sense to introduce a requires clause when the parent solvable
         // is undecided or going to be installed
         assert_ne!(decision_tracker.assigned_value(parent), Some(false));
 
-        let kind = Clause::Requires(parent, requirement);
-        let mut candidates = candidates.into_iter().peekable();
-        let first_candidate = candidates.peek().copied();
-        if let Some(first_candidate) = first_candidate {
-            match candidates.find(|&c| decision_tracker.assigned_value(c) != Some(false)) {
-                // Watch any candidate that is not assigned to false
-                Some(watched_candidate) => (
-                    kind,
-                    Some([parent.negative(), watched_candidate.positive()]),
-                    false,
-                ),
+        let kind = Clause::Requires(parent, condition.map(|d| d.0), requirement);
 
-                // All candidates are assigned to false! Therefore, the clause conflicts with the
-                // current decisions. There are no valid watches for it at the moment, but we will
-                // assign default ones nevertheless, because they will become valid after the solver
-                // restarts.
-                None => (
-                    kind,
-                    Some([parent.negative(), first_candidate.positive()]),
-                    true,
-                ),
+        // Construct literals to watch
+        let condition_literals = condition
+            .into_iter()
+            .flat_map(|(_, candidates)| candidates)
+            .copied()
+            .peekable();
+        let candidate_literals = candidates
+            .into_iter()
+            .map(|candidate| candidate.positive())
+            .peekable();
+
+        let mut literals = condition_literals.chain(candidate_literals).peekable();
+        let Some(&first_literal) = literals.peek() else {
+            // If there are no candidates and there is no condition, then this clause is an
+            // assertion.
+            // There is also no conflict because we asserted above that the parent is not
+            // assigned to false yet.
+            return (kind, None, false);
+        };
+
+        match literals.find(|&c| c.eval(decision_tracker.map()) != Some(false)) {
+            // Watch any candidate that is not assigned to false
+            Some(watched_candidate) => (kind, Some([parent.negative(), watched_candidate]), false),
+
+            // All candidates are assigned to false! Therefore, the clause conflicts with the
+            // current decisions. There are no valid watches for it at the moment, but we will
+            // assign default ones nevertheless, because they will become valid after the solver
+            // restarts.
+            None => {
+                // Try to find a condition that is not assigned to false.
+                (kind, Some([parent.negative(), first_literal]), true)
             }
-        } else {
-            // If there are no candidates there is no need to watch anything.
-            (kind, None, false)
         }
     }
 
@@ -230,6 +249,11 @@ impl Clause {
         )
     }
 
+    fn any_of(selected_var: VariableId, other_var: VariableId) -> (Self, Option<[Literal; 2]>) {
+        let kind = Clause::AnyOf(selected_var, other_var);
+        (kind, Some([selected_var.positive(), other_var.negative()]))
+    }
+
     /// Tries to fold over all the literals in the clause.
     ///
     /// This function is useful to iterate, find, or filter the literals in a
@@ -242,6 +266,7 @@ impl Clause {
             Vec<Vec<VariableId>>,
             ahash::RandomState,
         >,
+        disjunction_to_candidates: &Arena<DisjunctionId, Disjunction>,
         init: C,
         mut visit: F,
     ) -> ControlFlow<B, C>
@@ -255,14 +280,22 @@ impl Clause {
                 .iter()
                 .copied()
                 .try_fold(init, visit),
-            Clause::Requires(solvable_id, match_spec_id) => iter::once(solvable_id.negative())
-                .chain(
-                    requirements_to_sorted_candidates[&match_spec_id]
-                        .iter()
-                        .flatten()
-                        .map(|&s| s.positive()),
-                )
-                .try_fold(init, visit),
+            Clause::Requires(solvable_id, disjunction, match_spec_id) => {
+                iter::once(solvable_id.negative())
+                    .chain(
+                        disjunction
+                            .into_iter()
+                            .flat_map(|d| disjunction_to_candidates[d].literals.iter())
+                            .copied(),
+                    )
+                    .chain(
+                        requirements_to_sorted_candidates[&match_spec_id]
+                            .iter()
+                            .flatten()
+                            .map(|&s| s.positive()),
+                    )
+                    .try_fold(init, visit)
+            }
             Clause::Constrains(s1, s2, _) => [s1.negative(), s2.negative()]
                 .into_iter()
                 .try_fold(init, visit),
@@ -272,6 +305,9 @@ impl Clause {
             Clause::Lock(_, s) => [s.negative(), VariableId::root().negative()]
                 .into_iter()
                 .try_fold(init, visit),
+            Clause::AnyOf(selected, variable) => [selected.positive(), variable.negative()]
+                .into_iter()
+                .try_fold(init, visit),
         }
     }
 
@@ -286,11 +322,13 @@ impl Clause {
             Vec<Vec<VariableId>>,
             ahash::RandomState,
         >,
+        disjunction_to_candidates: &Arena<DisjunctionId, Disjunction>,
         mut visit: impl FnMut(Literal),
     ) {
         self.try_fold_literals(
             learnt_clauses,
             requirements_to_sorted_candidates,
+            disjunction_to_candidates,
             (),
             |_, lit| {
                 visit(lit);
@@ -348,12 +386,14 @@ impl WatchedLiterals {
         candidate: VariableId,
         requirement: Requirement,
         matching_candidates: impl IntoIterator<Item = VariableId>,
+        condition: Option<(DisjunctionId, &[Literal])>,
         decision_tracker: &DecisionTracker,
     ) -> (Option<Self>, bool, Clause) {
         let (kind, watched_literals, conflict) = Clause::requires(
             candidate,
             requirement,
             matching_candidates,
+            condition,
             decision_tracker,
         );
 
@@ -419,6 +459,11 @@ impl WatchedLiterals {
         (Self::from_kind_and_initial_watches(watched_literals), kind)
     }
 
+    pub fn any_of(selected_var: VariableId, other_var: VariableId) -> (Option<Self>, Clause) {
+        let (kind, watched_literals) = Clause::any_of(selected_var, other_var);
+        (Self::from_kind_and_initial_watches(watched_literals), kind)
+    }
+
     fn from_kind_and_initial_watches(watched_literals: Option<[Literal; 2]>) -> Option<Self> {
         let watched_literals = watched_literals?;
         debug_assert!(watched_literals[0] != watched_literals[1]);
@@ -437,6 +482,7 @@ impl WatchedLiterals {
             Vec<Vec<VariableId>>,
             ahash::RandomState,
         >,
+        disjunction_to_candidates: &Arena<DisjunctionId, Disjunction>,
         decision_map: &DecisionMap,
         for_watch_index: usize,
     ) -> Option<Literal> {
@@ -453,6 +499,7 @@ impl WatchedLiterals {
                 let next = clause.try_fold_literals(
                     learnt_clauses,
                     requirement_to_sorted_candidates,
+                    disjunction_to_candidates,
                     (),
                     |_, lit| {
                         // The next unwatched variable (if available), is a variable that is:
@@ -570,13 +617,14 @@ impl<I: Interner> Display for ClauseDisplay<'_, I> {
                 )
             }
             Clause::Learnt(learnt_id) => write!(f, "Learnt({learnt_id:?})"),
-            Clause::Requires(variable, requirement) => {
+            Clause::Requires(variable, condition, requirement) => {
                 write!(
                     f,
-                    "Requires({}({:?}), {})",
+                    "Requires({}({:?}), {}, condition={:?})",
                     variable.display(self.variable_map, self.interner),
                     variable,
                     requirement.display(self.interner),
+                    condition
                 )
             }
             Clause::Constrains(v1, v2, version_set_id) => {
@@ -611,6 +659,16 @@ impl<I: Interner> Display for ClauseDisplay<'_, I> {
                     other,
                 )
             }
+            Clause::AnyOf(variable, other) => {
+                write!(
+                    f,
+                    "AnyOf({}({:?}), {}({:?}))",
+                    variable.display(self.variable_map, self.interner),
+                    variable,
+                    other.display(self.variable_map, self.interner),
+                    other,
+                )
+            }
         }
     }
 }
@@ -664,6 +722,7 @@ mod test {
             parent,
             VersionSetId::from_usize(0).into(),
             [candidate1, candidate2],
+            None,
             &decisions,
         );
         assert!(!conflict);
@@ -687,6 +746,7 @@ mod test {
             parent,
             VersionSetId::from_usize(0).into(),
             [candidate1, candidate2],
+            None,
             &decisions,
         );
         assert!(!conflict);
@@ -710,6 +770,7 @@ mod test {
             parent,
             VersionSetId::from_usize(0).into(),
             [candidate1, candidate2],
+            None,
             &decisions,
         );
         assert!(conflict);
@@ -731,6 +792,7 @@ mod test {
                 parent,
                 VersionSetId::from_usize(0).into(),
                 [candidate1, candidate2],
+                None,
                 &decisions,
             )
         })
