@@ -15,6 +15,7 @@ use crate::*;
 #[derive(Debug, Clone)]
 pub struct InMemoryFile {
   sys: InMemorySys,
+  path: PathBuf,
   inner: Arc<RwLock<FileInner>>,
   pos: usize,
 }
@@ -615,6 +616,19 @@ impl BaseFsHardLink for InMemorySys {
   }
 }
 
+impl BaseFsCreateJunction for InMemorySys {
+  fn base_fs_create_junction(
+    &self,
+    _original: &Path,
+    _junction: &Path,
+  ) -> io::Result<()> {
+    Err(std::io::Error::new(
+      ErrorKind::Unsupported,
+      "Junctions are not supported in an in-memory sys.",
+    ))
+  }
+}
+
 #[derive(Debug, Clone)]
 pub struct InMemoryMetadata {
   file_type: FileType,
@@ -789,6 +803,7 @@ impl BaseFsOpen for InMemorySys {
           Ok(InMemoryFile {
             sys: self.clone(),
             inner: f.inner.clone(),
+            path,
             pos: if options.append {
               f.inner.read().data.len()
             } else {
@@ -816,6 +831,7 @@ impl BaseFsOpen for InMemorySys {
         let result = InMemoryFile {
           sys: self.clone(),
           inner: new_file.inner.clone(),
+          path,
           pos: if options.append {
             new_file.inner.read().data.len()
           } else {
@@ -846,7 +862,7 @@ impl BaseFsReadDir for InMemorySys {
     &self,
     path: &Path,
   ) -> std::io::Result<
-    Box<dyn Iterator<Item = std::io::Result<Self::ReadDirEntry>> + '_>,
+    Box<dyn Iterator<Item = std::io::Result<Self::ReadDirEntry>>>,
   > {
     let inner = self.0.read();
     let abs_path = inner.to_absolute_path(path);
@@ -1351,6 +1367,15 @@ impl FsFileSetLen for InMemoryFile {
   }
 }
 
+impl FsFileMetadata for InMemoryFile {
+  fn fs_file_metadata(&self) -> std::io::Result<BoxedFsMetadataValue> {
+    self
+      .sys
+      .base_fs_metadata(&self.path)
+      .map(BoxedFsMetadataValue::new)
+  }
+}
+
 impl FsFileSetPermissions for InMemoryFile {
   fn fs_file_set_permissions(&mut self, mode: u32) -> std::io::Result<()> {
     let mut inner = self.inner.write();
@@ -1631,6 +1656,12 @@ mod tests {
     let now = SystemTime::now();
     let duration = now.duration_since(modified);
     assert!(duration.is_ok());
+
+    let opened_file = sys.fs_open(file_path, &OpenOptions::new_read()).unwrap();
+    assert_eq!(
+      opened_file.fs_file_metadata().unwrap().modified().unwrap(),
+      modified
+    );
   }
 
   #[test]
