@@ -141,6 +141,13 @@ impl EnvHomeDir for RealSys {
   }
 }
 
+#[cfg(all(target_os = "windows", feature = "winapi"))]
+impl EnvProgramsDir for RealSys {
+  fn env_programs_dir(&self) -> Option<PathBuf> {
+    known_folder(&windows_sys::Win32::UI::Shell::FOLDERID_UserProgramFiles)
+  }
+}
+
 /// Uses the provided env for environment variables, but falls
 /// back to real sys calls.
 #[cfg(any(
@@ -321,6 +328,35 @@ impl BaseFsHardLink for RealSys {
   }
 }
 
+impl BaseFsCreateJunction for RealSys {
+  fn base_fs_create_junction(
+    &self,
+    original: &Path,
+    junction: &Path,
+  ) -> io::Result<()> {
+    #[cfg(all(target_os = "windows", feature = "winapi"))]
+    {
+      junction::create(original, junction)
+    }
+    #[cfg(all(target_os = "windows", not(feature = "winapi")))]
+    {
+      _ = (original, junction);
+      Err(Error::new(
+        ErrorKind::Unsupported,
+        "Enable the 'winapi' feature in sys_traits for junction support.",
+      ))
+    }
+    #[cfg(unix)]
+    {
+      _ = (original, junction);
+      Err(Error::new(
+        ErrorKind::Unsupported,
+        "Creating NTFS junctions is not supported on this platform",
+      ))
+    }
+  }
+}
+
 macro_rules! unix_metadata_prop {
   ($id:ident, $type:ident) => {
     #[inline]
@@ -615,7 +651,7 @@ impl BaseFsReadDir for RealSys {
     &self,
     path: &Path,
   ) -> std::io::Result<
-    Box<dyn Iterator<Item = std::io::Result<Self::ReadDirEntry>> + '_>,
+    Box<dyn Iterator<Item = std::io::Result<Self::ReadDirEntry>>>,
   > {
     let iterator = fs::read_dir(path)?;
     Ok(Box::new(iterator.map(|result| result.map(RealFsDirEntry))))
@@ -763,6 +799,12 @@ impl BaseFsWrite for RealSys {
 #[derive(Debug)]
 pub struct RealFsFile(fs::File);
 
+impl RealFsFile {
+  pub fn from_raw(file: fs::File) -> Self {
+    Self(file)
+  }
+}
+
 impl FsFile for RealFsFile {}
 
 impl FsFileAsRaw for RealFsFile {
@@ -924,6 +966,16 @@ impl FsFileSetLen for RealFsFile {
   }
 }
 
+impl FsFileMetadata for RealFsFile {
+  #[inline]
+  fn fs_file_metadata(&self) -> io::Result<BoxedFsMetadataValue> {
+    self
+      .0
+      .metadata()
+      .map(|m| BoxedFsMetadataValue::new(RealFsMetadata(m)))
+  }
+}
+
 impl FsFileSetPermissions for RealFsFile {
   #[inline]
   fn fs_file_set_permissions(&mut self, mode: u32) -> Result<()> {
@@ -1055,6 +1107,15 @@ mod test {
     assert!(RealSys.env_home_dir().is_some());
   }
 
+  #[cfg(all(target_os = "windows", feature = "winapi"))]
+  #[test]
+  fn test_known_folder_programs_dir() {
+    // was failing on gh actions for some reason
+    if std::env::var_os("CI").is_none() {
+      assert!(RealSys.env_programs_dir().is_some());
+    }
+  }
+
   #[cfg(all(unix, feature = "libc"))]
   #[test]
   fn test_umask() {
